<?php

class bdPaygateRuKassa_Processor_RuKassa extends bdPaygate_Processor_Abstract
{
	const CURRENCY_RUB = 'rub';

	const CURRENCY_UAH = 'uah';

	/**
	 * @return array|string[]
	 */
	public function getSupportedCurrencies()
	{
		return [
			bdPaygate_Processor_Abstract::CURRENCY_USD,
			bdPaygate_Processor_Abstract::CURRENCY_EUR,
			self::CURRENCY_RUB,
			self::CURRENCY_UAH
		];
	}

	/**
	 * @return bool
	 */
	public function isAvailable()
	{
		$options = XenForo_Application::getOptions();

		return !empty($options->bdPaygateRuKassa_ShopId)
			&& !empty($options->bdPaygateRuKassa_Token)
			&& !$this->_sandboxMode();
	}

	/**
	 * @return bool
	 */
	public function isRecurringSupported()
	{
		return false;
	}

	/**
	 * @param Zend_Controller_Request_Http $request
	 * @param string                       $transactionId
	 * @param string                       $paymentStatus
	 * @param string                       $transactionDetails
	 * @param string                       $itemId
	 *
	 * @return bool
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function validateCallback(
		Zend_Controller_Request_Http $request,
		&$transactionId,
		&$paymentStatus,
		&$transactionDetails,
		&$itemId
	)
	{
		$amount = false;
		$currency = false;

		return $this->validateCallback2($request, $transactionId, $paymentStatus, $transactionDetails, $itemId, $amount,
			$currency);
	}

	/**
	 * @param Zend_Controller_Request_Http $request
	 * @param string                       $transactionId
	 * @param string                       $paymentStatus
	 * @param string                       $transactionDetails
	 * @param string                       $itemId
	 * @param string                       $amount
	 * @param string                       $currency
	 *
	 * @return bool
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function validateCallback2(
		Zend_Controller_Request_Http $request,
		&$transactionId,
		&$paymentStatus,
		&$transactionDetails,
		&$itemId,
		&$amount,
		&$currency
	)
	{
		$input = new XenForo_Input($request);

		$params = [
			'signature'   => $_SERVER['HTTP_SIGNATURE'],
			'id'    	  => $input->filterSingle('id', XenForo_Input::STRING),
			'amount'      => $input->filterSingle('amount', XenForo_Input::STRING),
			'in_amount'   => $input->filterSingle('in_amount', XenForo_Input::STRING),
			'order_id'    => $input->filterSingle('order_id', XenForo_Input::STRING),
			'data'    	  => $input->filterSingle('data', XenForo_Input::STRING),
			'status'      => $input->filterSingle('status', XenForo_Input::STRING),
			
			//'merchantId'  => $input->filterSingle('merchant', XenForo_Input::STRING),
			//'item'        => $input->filterSingle('custom_field', XenForo_Input::STRING),
			//'intid'       => $input->filterSingle('intid', XenForo_Input::STRING)
		];

		$currency = self::CURRENCY_RUB;

		if (!empty($params))
		{
			$transactionDetails = $params;
			$options = XenForo_Application::get('options');
			if (!empty($options->bdPaygateRuKassa_ShopId) && !empty($options->bdPaygateRuKassa_Token) && !empty($params['signature']))
			{
				$signEquals1 = hash_equals($params['signature'], $this->getSignature([
					'id'  				=> $params['id'],
					'createdDateTime' 	=> $params['createdDateTime'],
					'amount'  			=> $params['amount']
				], $options->bdPaygateRuKassa_Token));


				if ($signEquals1)
				{
					$amount = $params['amount'];
					$in_amount = $params['in_amount'];

					if($in_amount < $amount) {
						$this->_setError('Not enough amount');

						echo $this->getResponse($this->getLastError(), 'error');

						return false;
					}

					$transactionId = 'rukassa_' . $params['id'];

					$processorModel = $this->getModelFromCache('bdPaygate_Model_Processor');

					// Проверяем, не была ли уже проведена такая операция
					$log = $processorModel->getLogByTransactionId($transactionId);
					if (!empty($log))
					{
						if (($log['log_type'] == 'accepted') || ($log['log_type'] == 'rejected') || ($log['log_type'] == 'error'))
						{
							$this->_setError("Transaction {$transactionId} has already been processed");

							echo $this->getResponse($this->getLastError(), 'error');

							return false;
						}
					}
					else
					{
						
						$data = json_decode($params['data'], true);
						
						$itemId = base64_decode($data['item_id']);

						$paymentStatus = bdPaygate_Processor_Abstract::PAYMENT_STATUS_ACCEPTED;

						echo $this->getResponse('Payment received.');

						return true;
					}
				}

				$this->_setError('Invalid signature');

				echo $this->getResponse($this->getLastError(), 'error');

				return false;
			}

			$this->_setError('Invalid public_key or secret_key.');

			echo $this->getResponse($this->getLastError(), 'error');

			return false;
		}

		$this->_setError('Callback not validated.');

		echo $this->getResponse($this->getLastError(), 'error');

		return false;
	}

	/**
	 * @param float  $amount
	 * @param string $currency
	 * @param string $itemName
	 * @param string $itemId
	 * @param bool   $recurringInterval
	 * @param bool   $recurringUnit
	 * @param array  $extraData
	 *
	 * @return string
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function generateFormData($amount, $currency, $itemName, $itemId, $recurringInterval = false, $recurringUnit = false, array $extraData = [])
	{
		$this->_assertAmount($amount);
		$this->_assertCurrency($currency);
		$this->_assertItem($itemName, $itemId);
		$this->_assertRecurring($recurringInterval, $recurringUnit);

		$options = XenForo_Application::get('options');

		$formAction = 'https://lk.rukassa.is/api/v1/create';

		$data = json_encode([
			'item_name' => $itemName,
			'item_id'	=> base64_encode($itemId),
			//'currency'	=> mb_strtoupper($currency),
		]);

		$payment = [
			'shop_id'     => $options->bdPaygateRuKassa_ShopId,
			'amount'      => number_format($amount, 2, '.', ''),
			'order_id'    => base64_encode(md5(time() . XenForo_Visitor::getUserId())),
			'token'       => $options->bdPaygateRuKassa_Token,
			'data'        => $data,
			'success_url' => $this->_generateReturnUrl($extraData),
			'fail_url'    => $this->_generateDetailUrl($extraData)
		];

		//$payment['s'] = $this->getSignature($payment, $options->bdPaygateRuKassa_SecretKey);

		$ch = curl_init($formAction);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($payment, '', '&'));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_HEADER, false);
		$result = json_decode(curl_exec($ch));
		curl_close($ch);

		$url = $result->url;

		// Генерация формы
		//$form = "<form action=\"{$formAction}\" target=\"blank\" method=\"GET\">";

		$callToAction = new XenForo_Phrase('bdpaygate_rukassa_call_to_action');
		/*
		foreach ($payment as $item => $value)
		{
			$form .= "<input type=\"hidden\" name=\"$item\" value=\"$value\" />";
		}

		$form .= "<input type=\"submit\" value=\"{$callToAction}\" class=\"button\"/></form><br />";

		return $form;*/
		
		return '<a href="' . $url . '" class="button">' . $callToAction . '</a>';
		
		
	}

	/**
	 * @param $params
	 * @param $token
	 *
	 * @return string
	 */
	protected function getSignature($params, $token)
	{
		return hash_hmac('sha256', $params['id'] . '|' . $params['createdDateTime'] . '|' . $params['amount'], $token);

		/*return hash('md5', implode(':', [
			$params['m'],
			$params['oa'],
			$key,
			$params['o']
		]));*/
	}

	/**
	 * @param string $message
	 * @param string $type
	 *
	 * @return false|string
	 */
	protected function getResponse($message = '', $type = 'result')
	{
		return json_encode([
			$type => [
				'message' => $message
			]
		]);
	}
}